#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vowel - AptWorker.py

import gtk

import os
import sys
import time
import tempfile
import thread
import threading
import subprocess
from subprocess import Popen, PIPE
import aptsources
from aptsources.sourceslist import SourcesList, SourceEntry


class AptWorker:
    
    (INSTALL, UPDATE) = range(2)
    
    def __init__(self):
        self.sourceslist = SourcesList()
        self.reload_sourceslist()
        self.canStart = True
        
    def reload_sourceslist(self):
        self.sourceslist.refresh()
        
    def write_sourceslist(self, lines, file=None):
        lines = lines.split('\n')
        for line in lines:
            entry = SourceEntry(line)
            self.sourceslist.add(entry.type, entry.uri, entry.dist, entry.comps, comment=entry.comment,
                                 pos=len(self.sourceslist.list), file=file)
        self.save_sourceslist()
        print 'OK'
    
    def save_sourceslist(self):
        self.sourceslist.backup('.save')
        self.sourceslist.save()
        self.reload_sourceslist()
    
    def add_key(self, keys):
        keys = keys.split('\n')
        for key in keys:
            os.system("wget -q " + key + " -O- | sudo apt-key add -")

    def run_vowel(self, lock, repo=None, key=None):
        cmd = []
        if os.getuid() != 0:
            cmd = ["/usr/bin/gksu",
                   "--desktop", "/usr/share/applications/vowel.desktop",
                   "--"]
            
        cmd += ["/usr/bin/vowel",
                "--write-sourceslist",
                "%s" % '\n'.join(repo)]
        
        key_length = len(key)
        count = 0
        
        if key_length != 0:
            cmd.append("--add-key")
            cmd.append("%s" % '\n'.join(key))
        p = subprocess.Popen(cmd, stdout=PIPE)
        str = p.communicate()[0].strip().split('\n')
        #print self.canStart
        for s in str:
            if s == 'OK':
                count+=1
        if count != key_length + 1:
            self.canStart = False
        #print self.canStart
        lock.release()
        
    def prepare(self, window_main, repo=None, key=None):
        #window_main.set_sensitive(False)
        window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        lock = threading.Lock()
        lock.acquire()
        t = threading.Thread(target=self.run_vowel,args=(lock,repo,key))
        t.start()
        while lock.locked():
            while gtk.events_pending():
                gtk.main_iteration()
            #time.sleep(0.05)
        window_main.window.set_cursor(None)
        #window_main.set_sensitive(True)
                    
    def run_synaptic(self, id, lock, to_add=None, to_rm=None, action=INSTALL):
        """ Run the synaptic program for installing packages or updating sourceslist """
        #apt_pkg.PkgSystemUnLock()
        #print "run_synaptic(%s,%s,%s)" % (id, lock, selec
            #print repotions)
        cmd = []
        if os.getuid() != 0:
            cmd = ["/usr/bin/gksu",
                   "--desktop", "/usr/share/applications/synaptic.desktop",
                   "--"]
        cmd += ["/usr/sbin/synaptic",
                "--hide-main-window",
                "--non-interactive",
                "--parent-window-id", "%s" % (id)]

        # create tempfile for install (here because it must survive
        # durng the synaptic call
        f = tempfile.NamedTemporaryFile()
        if action == self.INSTALL:
            # install the stuff
            for item in to_add:
                f.write("%s\tinstall\n" % item)
                #print item.pkgname
            for item in to_rm:
                f.write("%s\tuninstall\n" % item)
            cmd.append("--set-selections-file")
            cmd.append("%s" % f.name)
            f.flush()
            subprocess.call(cmd)
            f.close()
        elif action == self.UPDATE:
            #print "Updating..."
            cmd.append("--update-at-startup")
            subprocess.call(cmd)
        else:
            print "run_synaptic() called with unknown action"
            sys.exit(1)
        lock.release()
        
    def perform_action(self, window_main, cache, to_add=None, to_rm=None, action=INSTALL):
        """ Perform action installing packages provided by repositories """
        window_main.set_sensitive(False)
        window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        lock = threading.Lock()
        lock.acquire()
        t = threading.Thread(target=self.run_synaptic,args=(window_main.window.xid,lock,to_add,to_rm,action))
        t.start()
        while lock.locked():
            while gtk.events_pending():
                gtk.main_iteration()
            time.sleep(0.05)
        window_main.window.set_cursor(None)
        window_main.set_sensitive(True)
